@@ -36,7 +36,7 @@ my %red = qw(-bg red -fg white);
 my %green = qw(-bg green -fg white);
 my %white = qw(-fg black);
 
-my @hlb23ops = ($Tk::HListbox::VERSION >= 2.3) ? ('-font', 'Helvetica -11 bold', '-ipady', 1, '-tpady', 1) : ();
+my @hlb23ops = ($Tk::HListbox::VERSION >= 2.3) ? ('-font', 'Helvetica -11 bold', '-ipady', 1, '-tpady', 3) : ();
 my $ml = $mw->Scrolled('HMListbox',
     -scrollbars => 'osoe',
     -background => 'white', 
@@ -105,7 +105,7 @@ $ml->bindRows("<Double-Button-1>", \&openFileOrDir);
 $ml->bindColumns("<Button-3>", [\&columnPopup]);
 
 
-$ml->bindRows('<ButtonRelease-1>',  
+$ml->bindRows('<ButtonPress-1>',  
     sub {
         my ($w, $infoHR) = @_;
         print "You selected row: " . $infoHR->{-row} .
